﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace Task
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        BitmapSource lunarLanderImg;
        Lander lander;
        DispatcherTimer dt;
        Rectangle landingPoint;
        double sizeRatio;

        public MainWindow()
        {
            InitializeComponent();

            Activate();

            lunarLanderImg = new BitmapImage(new Uri("lunar_lander.png", UriKind.Relative));
            lander = new Task.Lander(new ImageBrush(lunarLanderImg), new Point(50, 50), new Size(20, 20), myCanvas, 1);

            var startingParams = InitializeLanding();

            landingPoint = new Rectangle();
            landingPoint.Fill = Brushes.Black;
            myCanvas.Children.Add(landingPoint);

            dt = new DispatcherTimer();
            dt.Interval = new TimeSpan(0, 0, 0, 0, 12);
            dt.Tick += Dt_Tick;
            dt.Start();
        }


        bool isDead = false;
        private void Dt_Tick(object sender, EventArgs e)
        {
            string enginesState = "nahoru=" + (lander.BottomEngine ? "1" : "0") + "&doleva=" + (lander.RightEngine ? "1" : "0") + "&doprava=" + (lander.LeftEngine ? "1" : "0");
            HttpWebRequest req = (HttpWebRequest)WebRequest.Create(new Uri(Settings.Uri, "/stav?id=" + Settings.Id + "&" + enginesState));


            using (WebResponse response = req.GetResponse())
            using (StreamReader sr = new StreamReader(response.GetResponseStream()))
            {
                string[] newPars = sr.ReadToEnd().Replace('.', ',').Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);

                if (newPars.Length == 1) // game over
                {
                    if (newPars[0] == "Zdar!")
                        MessageBox.Show("Přistání se zdařilo! Pro opětovný start klikněte na herní plochu");
                    else if (newPars[0] == "Krach!")
                        MessageBox.Show("Přistání se bohužel nezdařilo! Pro opětovný start klikněte na herní plochu");

                    isDead = true;
                    dt.Stop();
                    return;
                }

                string[] posMod = newPars[1].Split(' ');

                lander.Position = new Point(double.Parse(posMod[0]), double.Parse(posMod[1]));

                string[] velocity = newPars[2].Split(' ');

                lander.Velocity = new Vector(double.Parse(velocity[0]), double.Parse(velocity[1]));

            }

            double differenceX = (lander.Position.X - Settings.LandingPoint.X);
            double differenceY = Math.Abs(lander.Position.Y - Settings.LandingPoint.Y);

            Console.WriteLine(differenceY);


            double ratioX = myCanvas.ActualWidth / Math.Max(((Math.Abs(differenceX) + 15) * 2), 200);
            double ratioY = myCanvas.ActualHeight / Math.Max(differenceY + 30, 200);

            sizeRatio = Math.Min(ratioX, ratioY);



            lander.SizeRatio = sizeRatio;

            lander.RelativePosition = new Point(differenceX, differenceY);

            landingPoint.Width = 20 * sizeRatio;
            landingPoint.Height = 5 * sizeRatio;
            Canvas.SetLeft(landingPoint, myCanvas.ActualWidth / 2 - landingPoint.Width / 2);
            Canvas.SetTop(landingPoint, myCanvas.ActualHeight - 5);

        }

        Tuple<Vector, double> InitializeLanding()
        {
            Vector speed = new Vector();
            double fuel = 0;

            HttpWebRequest req = (HttpWebRequest)WebRequest.Create(new Uri(Settings.Uri, "/start"));

            using (WebResponse response = req.GetResponse())
            using (StreamReader sr = new StreamReader(response.GetResponseStream()))
            {
                string[] pars = sr.ReadToEnd().Replace('.', ',').Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);

                Settings.Id = uint.Parse(pars[0]);

                string[] posMod = pars[1].Split(' ');

                Settings.StartingPosition = new Point(double.Parse(posMod[0]), double.Parse(posMod[1]));

                string[] velocity = pars[2].Split(' ');

                speed = new Vector(double.Parse(velocity[0]), double.Parse(velocity[1]));

                fuel = double.Parse(pars[3]);

                string[] posLand = pars[4].Split(' ');
                Settings.LandingPoint = new Point((double.Parse(posLand[0]) + double.Parse(posLand[1])) / 2.0, 0);
            }

            return new Tuple<Vector, double>(speed, fuel);
        }

        private void Window_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.A:
                    lander.RightEngine = true;
                    break;

                case Key.W:
                    lander.BottomEngine = true;
                    break;

                case Key.D:
                    lander.LeftEngine = true;
                    break;
            }
        }

        private void Window_KeyUp(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.A:
                    lander.RightEngine = false;
                    break;

                case Key.W:
                    lander.BottomEngine = false;
                    break;

                case Key.D:
                    lander.LeftEngine = false;
                    break;
            }
        }

        private void myCanvas_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if (!isDead)
                return;

            InitializeLanding();
            dt.Start();
        }
    }
}
